//////////////////////////////////////////////////////
///////////////////  BLINKCTRL.ASM ///////////////////
////////////////  CREATED: 02.08.2011 ////////////////
/////////////////  AUTHOR: TOKYODRIFT ////////////////
//////////////////////////////////////////////////////



//////////////////////////////////////////////////////
// BlinkCtrl by TokyoDrift is licensed under a      //
// Creative Commons Attribution-NonCommercial-      //
// ShareAlike 3.0 Unported License. To view a copy  //
// of this license, visit                           //
// http://creativecommons.org/licenses/by-nc-sa/3.0 //
// Permissions beyond the scope of this license may //
// be available at http://tokyodrift-dev.info.      //
//////////////////////////////////////////////////////



//////////////////////////////////////////////////////
//////////////////////  DEFINES //////////////////////
//////////////////////////////////////////////////////
.include "tn25def.inc"
.equ F_CPU = 128000

//REGISTERS
.def TEMP1     = r16
.def TEMP2     = r17
.def MULT      = r18
.def MULT_CNT  = r19
.def PER       = r20
.def PER_CNT   = r21
.def MODE      = r22
.def DATA      = r23
.def DATA_ADR  = r24
.def DATA_MASK = r25

//IOs
.equ LEDDDR    = DDRB
.equ LEDPORT   = PORTB

//MODES
.equ BLINK     = 0
.equ SPI_MN1   = 1
.equ SPI_MN2   = 2
.equ SPI_R     = 3
.equ SPI_W     = 4

//CMDs
.equ READ      = 0
.equ WRITE     = 1
.equ REBOOT    = 2

//MISC
.equ MN1_R     = 0xDE
.equ MN2_R     = 0xAD
.equ MN1_T     = 0xBE
.equ MN2_T     = 0xEF



//////////////////////////////////////////////////////
////////////// ESEG : CONFIG VALUES //////////////////
//////////////////////////////////////////////////////
.eseg
.org 0x00   //PROBABLY UNNECESSARY
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 1
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 2
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 3
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 4
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 5
.db 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  //CHANNEL 6
.db 0x80   //MULT
.db 0xA0   //PER



//////////////////////////////////////////////////////
/////////////////// DSEG : RAM ///////////////////////
//////////////////////////////////////////////////////
.dseg
.org 0x00   //PROBABLY UNNECESSARY
.byte 120   //JUST SO THE ASSEMBLER KNOWS...



//////////////////////////////////////////////////////
//////////- CSEG : STARTS WITH INT-VECTOR ////////////
//////////////////////////////////////////////////////
.cseg
.org 0x00
   rjmp RESET
.org INT0addr
   reti
.org PCI0addr
   reti
.org OC1Aaddr
   reti
.org OVF1addr
   reti
.org OVF0addr
   reti
.org ERDYaddr
   reti
.org ACIaddr
   reti
.org ADCCaddr
   reti
.org OC1Baddr
   reti
.org OC0Aaddr
   rjmp TIMER_CMP_INT
.org OC0Baddr
   reti
.org WDTaddr
   reti
.org USI_STARTaddr
   reti
.org USI_OVFaddr
   reti
.org INT_VECTORS_SIZE



//////////////////////////////////////////////////////
//////////////// RESET : STARTUP CODE ////////////////
//////////////////////////////////////////////////////
RESET:
   //SPL=LOW(RAMEND) //only SPL present on T25
   ldi TEMP1, LOW(RAMEND)
   out SPL, TEMP1

   //LEDDDR=0xFF
   ldi TEMP1, 0xFF
   out LEDDDR, TEMP1

   //Z=0
   ldi ZL, 0
   ldi ZH, 0
RESET_LOAD:
   //(*Z)=EEPROM_READ(Z)
   rcall EEPROM_READ
   ST Z, TEMP1
   //IF((++Z)!=120)jmp LOAD
   inc ZL
   cpi ZL, 120
   brne RESET_LOAD

   //MULT=EEPROM_READ(Z++)
   rcall EEPROM_READ
   mov MULT, TEMP1
   inc ZL
   //PER=EEPROM_READ(Z)
   rcall EEPROM_READ
   mov PER, TEMP1

   //Prescaler 64, CTC Mode
   ldi TEMP1, (1<<WGM01)|(0<<WGM00)
   out TCCR0A, TEMP1
   ldi TEMP1, (0<<WGM02)|(0<<CS02)|(1<<CS01)|(1<<CS00)
   out TCCR0B, TEMP1
   //Compare Value 2; Enable interrupt
   ldi TEMP1, 2
   out OCR0A, TEMP1
   ldi TEMP1, OCIE1A
   out TIMSK, TEMP1

   //ENABLE INTERRUPTS AND JUMP TO LOOP
   sei
   rjmp MAIN



//////////////////////////////////////////////////////
//////////////// MAIN : ENDLESS LOOP /////////////////
//////////////////////////////////////////////////////
MAIN:
   //WHILE(1){}
   rjmp MAIN



//////////////////////////////////////////////////////
////////////// EEPROM_READ : TEMP1=(*Z) //////////////
//////////////////////////////////////////////////////
EEPROM_READ:
   //IF(EECR&EEWE!=0) jmp EEPROM_READ
   sbic EECR, EEPE
   rjmp EEPROM_READ

   //EEAR=Z, EECR|=(1<<EERE)
   out EEARH, ZH
   out EEARL, ZL
   sbi EECR, EERE
   
   //TEMP1=EEDR
   in TEMP1, EEDR
   ret



//////////////////////////////////////////////////////
//////////////// TIMER : 1ms TICK INT ////////////////
//////////////////////////////////////////////////////
TIMER_CMP_INT:
   //IF(MODE!=BLINK)return;
   cpi MODE, BLINK
   brne TIMER_CMP_INT_END

   //IF((++MULT_CNT)<MULT)return;
   inc MULT_CNT
   cp MULT_CNT, MULT
   brne TIMER_CMP_INT_END
   //MULT=0;
   ldi MULT_CNT, 0

   //IF((DATA_MASK<<=1)==0)DATA_MASK=1;
   lsl DATA_MASK
   brne TIMER_CMP_INT_MAIN
   ldi DATA_MASK, 1
   inc DATA_ADR

TIMER_CMP_INT_MAIN:
   //TEMP1=1, TEMP2=0, Y=DATA_ADR
   ldi TEMP1, 1
   ldi TEMP2, 0
   mov YL, DATA_ADR
   ldi YH, 0
TIMER_CMP_INT_LOOP:
   //DATA = (*Y)&DATA_MASK
   ld DATA, Y
   and DATA, DATA_MASK
   //IF(DATA!=0)TEMP2|=TEMP1
   cpi DATA, 0
   breq TIMER_CMP_INT_LOOP_
   or TEMP2, TEMP1
TIMER_CMP_INT_LOOP_:
   //Y+=20
   adiw Y, 20
   //IF((TEMP1<<=1)!=0b00100000)goto LOOP
   lsl TEMP1
   cpi TEMP1, 0b00100000
   brne TIMER_CMP_INT_LOOP
   //LEDPORT=TEMP2
   out LEDPORT, TEMP2

   //IF((++PER_CNT)!=PER)reti
   inc PER_CNT
   cp PER_CNT, PER
   brne TIMER_CMP_INT_END
   //PER_CNT=0, DATA_ADR=0, DATA_MASK=1
   ldi PER_CNT, 0
   ldi DATA_ADR, 0
   ldi DATA_MASK, 1
   
TIMER_CMP_INT_END:
   reti
